﻿namespace HeuristicLab.Algorithms.PushGP.Expressions {
  using HeuristicLab.Algorithms.PushGP.Interpreter;

  /// <summary>
  ///     Pushes the logical AND of the top two BOOLEANs.
  /// </summary>
  //[PushExpression("BOOLEAN", "AND")]
  public class BooleanAndExpression : PushResultExpression<bool> {
    public BooleanAndExpression() : base("BOOLEAN.AND") { }
    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.BooleanStack, 2, values => values[0] && values[1]);
    }
  }

  /// <summary>
  ///     Pushes the logical OR of the top two BOOLEANs.
  /// </summary>
  //[PushExpression("BOOLEAN", "OR")]
  public class BooleanOrExpression : PushResultExpression<bool> {
    public BooleanOrExpression() : base("BOOLEAN.OR") { }

    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.BooleanStack, 2, values => values[0] || values[1]);
    }
  }

  /// <summary>
  ///     Pushes the logical NOT of the top BOOLEAN.
  /// </summary>
  //[PushExpression("BOOLEAN", "NOT")]
  public class BooleanNotExpression : PushResultExpression<bool> {
    public BooleanNotExpression() : base("BOOLEAN.NOT") { }

    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter.BooleanStack, 1, values => !values[0]);
    }
  }

  /// <summary>
  ///     Pushes FALSE if the top FLOAT is 0.0, or TRUE otherwise.
  /// </summary>
  public class BooleanFromFloatExpression : StatelessExpression {
    public BooleanFromFloatExpression() : base("BOOLEAN.FROMFLOAT") { }

    public override void Eval(IPushGpInterpreter interpreter) {
      if (interpreter.FloatStack.Count == 0) return;

      var value = interpreter.FloatStack.Pop() != 0.0;

      interpreter.BooleanStack.Push(value);
    }
  }

  /// <summary>
  ///     Pushes FALSE if the top INTEGER is 0, or TRUE otherwise.
  /// </summary>
  public class BooleanFromIntegerExpression : StatelessExpression {
    public BooleanFromIntegerExpression() : base("BOOLEAN.FROMINTEGER") { }

    public override void Eval(IPushGpInterpreter interpreter) {
      if (interpreter.IntegerStack.Count == 0) return;

      var value = interpreter.IntegerStack.Pop() != 0;

      interpreter.BooleanStack.Push(value);
    }
  }
}