﻿namespace HeuristicLab.Algorithms.PushGP.Expressions {
  using HeuristicLab.Algorithms.PushGP.Interpreter;
  using HeuristicLab.Algorithms.PushGP.Stack;

  public abstract class DoTimesExpression : LoopExpression {
    protected DoTimesExpression(string stringRepresentation) : base(stringRepresentation) { }
    protected override bool HasInsufficientArguments(IPushGpInterpreter interpreter, IStack<Expression> sourceStack) {
      return (interpreter.IntegerStack.Count < 1) || (sourceStack.Count == 0) || (interpreter.IntegerStack.Top <= 0);
    }

    protected override void InitState(IPushGpInterpreter interpreter, IStack<Expression> sourceStack) {
      this.State.Body = sourceStack.Pop();
      this.State.DestinationIndex = interpreter.IntegerStack.Pop();
      this.State.CurrentIndex = 1;
      this.State.Incrementor = 1;
    }

    protected override void PushIteration(IPushGpInterpreter interpreter) {
      interpreter.ExecStack.Push(this, this.State.Body);
    }

    protected override void PushLastIteration(IPushGpInterpreter interpreter) {
      interpreter.ExecStack.Push(this.State.Body);
    }
  }

  /// <summary>
  ///     Like CODE.DO*COUNT but does not push the loop counter.
  ///     This should be implemented as a macro that expands into CODE.DO*RANGE, similarly to the implementation
  ///     of CODE.DO*COUNT, except that a call to INTEGER.POP should be tacked on to the front of the loop body code
  ///     in the call to CODE.DO*RANGE.
  /// </summary>
  public class CodeDoTimesExpression : DoTimesExpression {
    public CodeDoTimesExpression() : base("CODE.DO*TIMES") { }

    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter, interpreter.CodeStack);
    }
  }

  /// <summary>
  ///     Like EXEC.DO*COUNT but does not push the loop counter. This should be implemented as a macro that expands into
  ///     EXEC.DO*RANGE,
  ///     similarly to the implementation of EXEC.DO*COUNT, except that a call to INTEGER.POP should be tacked on to the
  ///     front of the
  ///     loop body code in the call to EXEC.DO*RANGE. This call to INTEGER.POP will remove the loop counter, which will have
  ///     been
  ///     pushed by EXEC.DO*RANGE, prior to the execution of the loop body.
  /// </summary>
  public class ExecDoTimesExpression : DoTimesExpression {
    public ExecDoTimesExpression() : base("EXEC.DO*TIMES") { }

    public override void Eval(IPushGpInterpreter interpreter) {
      this.Eval(interpreter, interpreter.ExecStack);
    }
  }
}