﻿namespace HeuristicLab.Algorithms.PushGP.Expressions
{
    using System.Linq;
    using System.Threading;
    using Data.Random;
    using Interpreter;
    using Core;

    public class NameDefineXExecExpression : WrapperExpression<string>
    {
        public NameDefineXExecExpression(string state) : base(state) { }

        public override void Eval(IPushGpInterpreter interpreter)
        {
            Expression expression;
            if (!interpreter.IsNameQuoteFlagSet &&
                interpreter.CustomExpressions.TryGetValue(State, out expression))
            {
                interpreter.ExecStack.Push(expression);
            }
            else
            {
                interpreter.NameStack.Push(State);
                interpreter.IsNameQuoteFlagSet = false;
            }
        }
    }

    /// <summary>
    ///     Sets a flag indicating that the next State encountered will be pushed onto the NAME stack (and not have its
    ///     associated value
    ///     pushed onto the EXEC stack), regardless of whether or not it has a definition. Upon encountering such a State and
    ///     pushing it
    ///     onto the NAME stack the flag will be cleared (whether or not the pushed State had a definition).
    /// </summary>
    public class NameQuoteExpression : StatelessExpression
    {
        public NameQuoteExpression() : base("NAME.QUOTE")
        {
        }

        public override void Eval(IPushGpInterpreter interpreter)
        {
            interpreter.IsNameQuoteFlagSet = true;
        }
    }

    /// <summary>
    ///     Pushes a randomly selected NAME that already has a definition.
    /// </summary>
    public class NameRandBoundNameExpression : StatelessExpression
    {
        private static readonly ThreadLocal<IRandom> Rand = RandomFactory.GetRandom();

        public NameRandBoundNameExpression() : base("NAME.RANDBOUNDNAME")
        {
        }

        public override void Eval(IPushGpInterpreter interpreter)
        {
            if (!interpreter.CustomExpressions.Any()) return;

            var index = Rand.Value.Next(0, interpreter.CustomExpressions.Keys.Count - 1);
            var state = interpreter.CustomExpressions.Keys.ElementAt(index);

            interpreter.NameStack.Push(state);
        }
    }
}