﻿namespace HeuristicLab.Algorithms.PushGP.Expressions {
  using Interpreter;
  using Stack;

  /// <summary>
  ///     Swaps the top two values.
  /// </summary>
  /// <typeparam name="T">Stacktype</typeparam>
  public abstract class SwapExpression<T> : StatelessExpression {
    protected SwapExpression(string stringRepresentation) : base(stringRepresentation) {
    }

    public void Eval(IStack<T> stack) {
      if (stack.Count < 2)
        return;

      stack.Swap(2);
    }
  }

  public class IntegerSwapExpression : SwapExpression<long> {
    public IntegerSwapExpression() : base("INTEGER.SWAP") {
    }

    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.IntegerStack);
    }
  }

  public class FloatSwapExpression : SwapExpression<double> {
    public FloatSwapExpression() : base("FLOAT.SWAP") {
    }

    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.FloatStack);
    }
  }

  public class BooleanSwapExpression : SwapExpression<bool> {
    public BooleanSwapExpression() : base("BOOLEAN.SWAP") {
    }

    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.BooleanStack);
    }
  }

  public class NameSwapExpression : SwapExpression<string> {
    public NameSwapExpression() : base("NAME.SWAP") {
    }

    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.NameStack);
    }
  }

  public class ExecSwapExpression : SwapExpression<Expression> {
    public ExecSwapExpression() : base("EXEC.SWAP") {
    }

    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.ExecStack);
    }
  }

  public class CodeSwapExpression : SwapExpression<Expression> {
    public CodeSwapExpression() : base("CODE.SWAP") {
    }

    public override void Eval(IPushGpInterpreter interpreter) {
      Eval(interpreter.CodeStack);
    }
  }
}